import grpc

from perxis.auth import APIKeyPlugin


class GrpcChannel:
    call_credentials: grpc.CallCredentials
    channel_credentials: grpc.ChannelCredentials
    composite_credentials: grpc.ChannelCredentials
    channel: grpc.Channel | None = None

    def __init__(
        self,
        target: str,
        metadata_plugin: grpc.AuthMetadataPlugin,
        project_name: str,
        project_version: str,
        call_credentials_name: str = "api-key",
    ) -> None:
        self.target = target
        self.metadata_plugin = metadata_plugin
        self.call_credentials_name = call_credentials_name
        self._init_credentials()
        self.connect(project_name, project_version)

    def _init_credentials(self) -> None:
        self.call_credentials = grpc.metadata_call_credentials(
            self.metadata_plugin, name=self.call_credentials_name
        )
        self.channel_credentials = grpc.ssl_channel_credentials()
        self.composite_credentials = grpc.composite_channel_credentials(
            self.channel_credentials, self.call_credentials
        )

    def connect(self, project_name: str, project_version: str) -> None:
        if not self.channel:
            self.channel = grpc.aio.secure_channel(
                self.target,
                self.composite_credentials,
                options=(
                    ("grpc.primary_user_agent", f"{project_name}/{project_version}"),
                ),
            )

    async def close(self) -> None:
        await self.channel.close()


def get_grpc_channel(
    target: str, api_key: str, project_name: str, project_version: str
) -> GrpcChannel:
    return GrpcChannel(
        target=target,
        metadata_plugin=APIKeyPlugin(token=api_key),
        project_name=project_name,
        project_version=project_version,
    )
